/**********************************************************************
 Freeciv - Copyright (C) 2005 - The Freeciv Team
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/ 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gentree.h"

#include "element.h"
#include "sprite_g.h"

/**************************************************************************

**************************************************************************/
struct element *element_new()
{
  struct element *pelement = fc_malloc(sizeof(*pelement));

  /* Fill element with sane values. */

  gentree_new(pelement);

  pelement->name    = NULL;
  pelement->x       = 0;
  pelement->y       = 0;
  pelement->w       = 0; 
  pelement->h       = 0;
  pelement->visible = FALSE;
  pelement->sprite  = NULL;
  pelement->text    = NULL;
  pelement->back    = FALSE;

  return pelement;
}

/**************************************************************************

**************************************************************************/
void element_draw(struct canvas *pcanvas, struct element *pelement, int x,
		  int y, int w, int h)
{
  struct element *parent, *child;
  int absx, absy;
  if (!pelement || !pelement->visible) {
    return;
  }

  /* Determine the actual coordinates of the element */
  absx = pelement->x;
  absy = pelement->y;

  parent = (void *)pelement->node.parent;
  while (parent) {
    absx += parent->x;
    absy += parent->y;
    parent = (void *)parent->node.parent;
  }

  /* Clip the rectangle to the element. */
  w = MIN(w + x, absx + pelement->w);
  h = MIN(h + y, absy + pelement->h);
  x = MAX(x, absx);
  y = MAX(y, absy);
  w -= x;
  h -= y;

  if (w <= 0 || h <= 0) {
    return;
  }
  
  if (pelement->back) {
    canvas_put_rectangle(pcanvas, COLOR_STD_BACKGROUND, x, y, w, h);
  }

  if (pelement->sprite) {
    canvas_put_sprite(pcanvas, x, y, pelement->sprite, x - absx, y - absy, w,
		      h);
  }
  if (pelement->text) {
    canvas_put_text(pcanvas, absx, absy, 1, COLOR_STD_WHITE, pelement->text);
  }
  child = (void *)pelement->node.fchild;
  while(child) {
    element_draw(pcanvas, child, x, y, w, h);
    child = (void *)child->node.rsibling;
  }
}

static struct Sprite *slate_sprite = NULL;

/**************************************************************************

**************************************************************************/
struct element *element_demo(int w, int h)
{
  struct element *pelement, *demo_element, *science_window;

  if (!slate_sprite) {
    slate_sprite = load_gfxfile("data/misc/slate.png");
  }

  pelement = element_new();
  pelement->name    = strdup("root_element");
  pelement->x       = 0;
  pelement->y       = 0;
  pelement->w       = w;
  pelement->h       = h;
  pelement->visible = TRUE;
  pelement->sprite  = NULL;
  pelement->text    = NULL;
  pelement->back    = FALSE;

  demo_element = pelement;

  pelement = element_new();
  pelement->name    = strdup("science_window");
  pelement->x       = 10;
  pelement->y       = 10;
  pelement->w       = 300;
  pelement->h       = 200;
  pelement->visible = TRUE;
  pelement->sprite  = slate_sprite;
  pelement->text    = strdup("Science Report");
  pelement->back    = FALSE;

  science_window = pelement;
  gentree_insert(demo_element, science_window);

  pelement = element_new();
  pelement->name    = strdup("known_label");
  pelement->x       = 10;
  pelement->y       = 40;
  pelement->w       = 80;
  pelement->h       = 20;
  pelement->visible = TRUE;
  pelement->sprite  = NULL;
  pelement->text    = strdup("Known");
  pelement->back    = TRUE;

  gentree_insert(science_window, pelement);

  pelement = element_new();
  pelement->name    = strdup("researching_label");
  pelement->x       = 110;
  pelement->y       = 40;
  pelement->w       = 80;
  pelement->h       = 20;
  pelement->visible = TRUE;
  pelement->sprite  = NULL;
  pelement->text    = strdup("Researching");
  pelement->back    = TRUE;

  gentree_insert(science_window, pelement);

  pelement = element_new();
  pelement->name    = strdup("goal_label");
  pelement->x       = 210;
  pelement->y       = 40;
  pelement->w       = 80;
  pelement->h       = 20;
  pelement->visible = TRUE;
  pelement->sprite  = NULL;
  pelement->text    = strdup("Goal");
  pelement->back    = TRUE;

  gentree_insert(science_window, pelement);

  return demo_element;
}

