/**********************************************************************
 Freeciv - Copyright (C) 2005 - The Freeciv Team
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/ 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>

#include "mem.h"

#include "gentree.h"
#include "log.h"

/**************************************************************************
  Blank the header of a node.
**************************************************************************/
void gentree_new(void *pvgentree)
{
  struct gentree_header *pgentree = pvgentree;

  pgentree->parent = NULL;
  pgentree->fchild = NULL;
  pgentree->lsibling = NULL;      
  pgentree->rsibling = NULL;
}

/**************************************************************************
  Add a child to this tree, containing the specified data.
**************************************************************************/
void gentree_insert(void *vparent, void *vchild)
{
  struct gentree_header *parent = vparent;
  struct gentree_header *child = vchild;

  child->parent = parent;
  child->fchild = NULL;
  child->lsibling = NULL;      
  child->rsibling = NULL;

  if (parent) {
    if (!parent->fchild) {
      parent->fchild = child;
    } else {
      struct gentree_header *sibling;

      sibling = parent->fchild;
      while (sibling->rsibling) {
	sibling = sibling->rsibling;
      }
      sibling->rsibling = child;
      child->lsibling = sibling;
    }
  }
}

/**************************************************************************
  Frees this node and all children.
**************************************************************************/
void gentree_free(void *pvgentree)
{
  struct gentree_header *pgentree = pvgentree;
  while (pgentree->fchild) {
    gentree_free(pgentree->fchild);
  }

  if (pgentree->parent) {
    if (!pgentree->lsibling) {
      pgentree->parent->fchild = pgentree->rsibling;
    } else {
      pgentree->lsibling->rsibling = pgentree->rsibling;
    }
    if (pgentree->rsibling) {
      pgentree->rsibling->lsibling = pgentree->lsibling;
    }
  }

  free(pgentree);
}

/**************************************************************************
  Execute the given function on all nodes in this tree, first on the head
  node, then, if the function returned true, on each child.
**************************************************************************/
void gentree_traverse(void *pvgentree,
		      bool (*func)(void *))
{
  struct gentree_header *pgentree = pvgentree;
  if (!pgentree) {
    return;
  }

  if (func(pgentree)) {
    struct gentree_header *child = pgentree->fchild;

    while(child) {
      gentree_traverse(child, func);
      child = child->rsibling;
    }
  }
}
